@@ -10,7 +10,7 @@ class LocalShareModeTimerTooShortTest(unittest.TestCase, GuiShareTest):
     def setUpClass(cls):
         test_settings = {
             "public_mode": False,
-            "shutdown_timeout": True,
+            "autostop_timer": True,
         }
         cls.gui = GuiShareTest.set_up(test_settings)
 
@@ -19,6 +19,7 @@ class LocalShareModeTimerTooShortTest(unittest.TestCase, GuiShareTest):
         GuiShareTest.tear_down()
 
     @pytest.mark.gui
+    @pytest.mark.skipif(pytest.__version__ < '2.9', reason="requires newer pytest")
     def test_gui(self):
         self.run_all_common_setup_tests()
         self.run_all_share_mode_setup_tests()
